#uses "fwUnitTestComponentAsserts.ctl"



/**@file test_fwCaen.ctl

  @brief This file is a library with the required functions to run fwCaen unit test for the Vertical Slice test bench

*/


/**@brief Prepare the appropiate global environment for all of the test cases in this suite.
This function is called only once before any test is launched.

It means:

   - Import dpList: Pilot OPC UA DPs, test DPs,...
   - Create managers
   - Configure OPC UA clients
   - Start managers

*/
void test_fwCaen_setupSuite()
{
  DebugTN("test_fwCaen_setupSuite() -> Launched...");

  const string OPC_UA_SETUP_FILE  = "OpcUaSetup";
  const string sVENUS_OPC_UA_FILE = "VenusCaenOpcUa";
  const string sPILOT_OPC_UA_FILE = "PilotCaenOpcUa";


  int iLoop, iLen;
  string sInstallationPath, sFileName;
  dyn_int diDriverNumber;
  dyn_string dsInternalDp, dsInternalDpt, exceptionInfo;


  // Create internal DPs
  DebugTN("test_fwCaen_setupSuite() -> Internal DPs -> Start...");
  dsInternalDp  = makeDynString("_Driver6",
                                "_Driver6_2",
                                "_Driver10",
                                "_Driver10_2",
                                "_Simulator6",
                                "_Simulator10",
                                "_Stat_driver_6_to_data_0",
                                "_Stat_driver_10_to_data_0",
                                "_Stat_Configs_driver_6",
                                "_Stat_2_Configs_driver_6",
                                "_Stat_Configs_driver_10",
                                "_Stat_2_Configs_driver_10");

  dsInternalDpt = makeDynString("_DriverCommon",
                                "_DriverCommon",
                                "_DriverCommon",
                                "_DriverCommon",
                                "_Simulator",
                                "_Simulator",
                                "_Statistics_Connections",
                                "_Statistics_Connections",
                                "_Statistics_DriverConfigs",
                                "_Statistics_DriverConfigs",
                                "_Statistics_DriverConfigs",
                                "_Statistics_DriverConfigs");
                                
  if( CreateInternalDPs(dsInternalDp, dsInternalDpt, exceptionInfo) == FALSE )
  {
    DebugTN("test_fwCaen_setupSuite() -> Error creating internal DPs: ", exceptionInfo);
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
  }
  DebugTN("test_fwCaen_setupSuite() -> Internal DPs -> Created OK");




  // Import OPC UA configuration
  DebugTN("test_fwCaen_setupSuite() -> Import OPC UA configuration -> Start...");

  sInstallationPath = getPath(DPLIST_REL_PATH, "", "", (SEARCH_PATH_LEN-1));
  sInstallationPath = sInstallationPath + "test/fwCaen/";

  sFileName = OPC_UA_SETUP_FILE;
  if( ImportDplFile(sInstallationPath, sFileName, exceptionInfo) == FALSE )
  {
    DebugTN("test_fwCaen_setupSuite() -> Import OPC UA configuration  -> Error in the process, details: ", exceptionInfo);
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
  }
  DebugTN("test_fwCaen_setupSuite() -> Import OPC UA configuration  -> Done OK");




  // Stop simulators, create OPC UA managers and start them
  DebugTN("test_fwCaen_setupSuite() -> Create OPC UA managers -> Start...");
  diDriverNumber = makeDynInt(6, 10);

  iLen = dynlen(diDriverNumber);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( CreateOpcUaManager(diDriverNumber[iLoop], exceptionInfo) == FALSE )
    {
      DebugTN("test_fwCaen_setupSuite() -> Error creating OPC UA manager: " + diDriverNumber[iLoop] + ", due to error: ", exceptionInfo);
      throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    }
    else
    {
      DebugTN("test_fwCaen_setupSuite() -> OPC UA manager with driver number: " + diDriverNumber[iLoop] + ", ready");
    }
  }
  DebugTN("test_fwCaen_setupSuite() -> Create OPC UA managers -> OK");


  // Done with ASCII import, in the future with JCOP code
    // Create OPC UA connection to CAEN Venus OPC UA server
    // Create OPC UA connection to Pilot OPC UA server
    // Instantiate CAEN Venus setup with fwDevice calls: Crates, Boards, Channels, Easy Crates, Easy Boards, Easy Channels

  DebugTN("test_fwCaen_setupSuite() -> Import Venus Caen OPC UA setup -> Start...");
  sFileName = sVENUS_OPC_UA_FILE;
  if( ImportDplFile(sInstallationPath, sFileName, exceptionInfo) == FALSE )
  {
    DebugTN("test_fwCaen_setupSuite() -> Import Venus Caen OPC UA setup -> Error in the process, details: ", exceptionInfo);
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
  }
  DebugTN("test_fwCaen_setupSuite() -> Import Venus Caen OPC UA setup -> Done OK");


  DebugTN("test_fwCaen_setupSuite() -> Import Pilot Caen OPC UA setup -> Start...");
  sFileName = sPILOT_OPC_UA_FILE;
  if( ImportDplFile(sInstallationPath, sFileName, exceptionInfo) == FALSE )
  {
    DebugTN("test_fwCaen_setupSuite() -> Import Pilot Caen OPC UA setup -> Error in the process, details: ", exceptionInfo);
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
  }
  DebugTN("test_fwCaen_setupSuite() -> Import Pilot Caen OPC UA setup -> Done OK");


  delay(45);


  // To be removed only when we will instantiate devices with JCOP devices, and Venus CAEN simulator has all defined properties to simulate  
    // Disable addressing of items not supported on simulator:
  /*
  int iLoop, iLen;
  dyn_bool dbEnableAddress;
  dyn_string dsDisableAddress;
  dyn_errClass err;

  dsDisableAddress = makeDynString("CAEN/develSY4527/branchController00/easyCrate0/easyBoard00.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard04.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard08.Information.Model",

                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard12.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard12/channel000.settings.outReg",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard12/channel000.readBackSettings.outReg",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard12/channel001.settings.outReg",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard12/channel001.readBackSettings.outReg",


                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard16.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard16/channel000.settings.outReg",
                                   "CAEN/develSY4527/branchController00/easyCrate0/easyBoard16/channel000.readBackSettings.outReg",

                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22.Actual.StatusIn",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22.Actual.Tracking",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel000.actual.intFail",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel000.actual.maxDrop",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel000.actual.remIlk",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel000.actual.temp",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel001.actual.intFail",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel001.actual.maxDrop",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel001.actual.remIlk",
                                   "CAEN/develSY4527/branchController00/easyCrate0/powerConverter22/channel001.actual.temp",

                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel000.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel000.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel000.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel000.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel001.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel001.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel001.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel001.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel002.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel002.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel002.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel002.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel003.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel003.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel003.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel003.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel004.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel004.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel004.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel004.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel005.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel005.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel005.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard00/channel005.readBackSettings.globalOnEn",

                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel000.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel000.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel000.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel000.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel001.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel001.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel001.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel001.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel002.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel002.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel002.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel002.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel003.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel003.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel003.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel003.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel004.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel004.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel004.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel004.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel005.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel005.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel005.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel005.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel006.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel006.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel006.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel006.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel007.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel007.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel007.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel007.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel008.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel008.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel008.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel008.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel009.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel009.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel009.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel009.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel010.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel010.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel010.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel010.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel011.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel011.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel011.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel011.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel012.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel012.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel012.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel012.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel013.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel013.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel013.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel013.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel014.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel014.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel014.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel014.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel015.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel015.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel015.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel015.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel016.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel016.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel016.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel016.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel017.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel017.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel017.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel017.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel018.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel018.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel018.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel018.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel019.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel019.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel019.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel019.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel020.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel020.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel020.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel020.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel021.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel021.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel021.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel021.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel022.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel022.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel022.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel022.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel023.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel023.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel023.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel023.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel024.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel024.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel024.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel024.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel025.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel025.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel025.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel025.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel026.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel026.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel026.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel026.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel027.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel027.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel027.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel027.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel028.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel028.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel028.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel028.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel029.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel029.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel029.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel029.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel030.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel030.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel030.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel030.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel031.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel031.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel031.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard03/channel031.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07.Information.Model",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel000.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel000.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel000.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel000.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel001.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel001.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel001.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel001.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel002.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel002.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel002.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel002.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel003.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel003.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel003.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel003.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel004.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel004.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel004.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel004.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel005.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel005.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel005.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel005.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel006.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel006.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel006.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel006.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel007.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel007.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel007.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel007.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel008.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel008.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel008.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel008.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel009.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel009.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel009.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel009.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel010.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel010.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel010.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel010.readBackSettings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel011.settings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel011.settings.globalOnEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel011.readBackSettings.globalOffEn",
                                   "CAEN/develSY4527/branchController00/easyCrate1/easyBoard07/channel011.readBackSettings.globalOnEn");


  iLen = dynlen(dsDisableAddress);
  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    dsDisableAddress[iLoop] = dsDisableAddress[iLoop] + ":_address.._active";
    dbEnableAddress [iLoop] = FALSE;
  }

  if( dpSet(dsDisableAddress, dbEnableAddress) != 0 )
  {
    DebugTN("Error disabling addresses");

    err = getLastError();
    if( dynlen(err) > 0 )
    {
      errorDialog(err);
      throwError(err); // write errors to stderr
    }
  }
  */



  // Check connection with CAEN OPC UA server


  // Check connection with PILOT OPC UA server

}






/**@brief Check if desired internal DPs exists, if not it create them


*/
bool CreateInternalDPs(dyn_string dsInternalDp, dyn_string dsInternalDpt, dyn_string &exceptionInfo)
{
  int iLoop, iLen, iRes;
  dyn_errClass err;


  iLen = dynlen(dsInternalDp);
  if( iLen != dynlen(dsInternalDpt) )
  {
    fwException_raise(exceptionInfo, "ERROR", "CreateInternalDPs() -> Different number of elements of DPs and DPTs", "");
    return FALSE;
  }


  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    if( dpExists(dsInternalDp[iLoop]) == FALSE )
    {
      DebugTN("CreateInternalDPs() -> " + iLoop + " / " + iLen + " -> Creating internal DP: " + dsInternalDp[iLoop] + ", [" + dsInternalDpt[iLoop] + "]");

      iRes = dpCreate(dsInternalDp[iLoop], dsInternalDpt[iLoop]);
      if( iRes != 0 )
      {
        fwException_raise(exceptionInfo, "ERROR", "CreateInternalDPs() -> Creating DP: " + dsInternalDp[iLoop] + ", [" + dsInternalDpt[iLoop] + "]", "");
        return FALSE;
      }
      else
      {
        err = getLastError();
        if( dynlen(err) > 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "CreateInternalDPs() -> Creating DP: " + dsInternalDp[iLoop] + ", [" + dsInternalDpt[iLoop] + "] -> Error code: " + getErrorCode(err) + ": " + getErrorText() , "");
          return FALSE;
        }
        else
        {
          DebugTN("CreateInternalDPs() -> " + iLoop + " / " + iLen + " -> Created internal DP: " + dsInternalDp[iLoop] + ", [" + dsInternalDpt[iLoop] + "] OK");
        }
      }

    }// DP exists
    else
    {
      DebugTN("CreateInternalDPs() -> " + iLoop + " / " + iLen + " -> internal DP: " + dsInternalDp[iLoop] + ", [" + dsInternalDpt[iLoop] + "] already exists");
    }

  }// For

  return TRUE;
}





bool CreateOpcUaManager(int iDriverNumber, dyn_string &exceptionInfo)
{
  bool bOpcUaExists, bSimulatorExists, bIsRunning;
  int iRes, iLen, iLoop;
  dyn_dyn_mixed ddmAllManagers;


  iRes = fwInstallationManager_getAllInfoFromPvss(ddmAllManagers);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Error getting existant managers information", "");
    return FALSE;
  }


  iLen = dynlen(ddmAllManagers);
  DebugTN("CreateOpcUaManagers() -> Retrieved all project managers (" + iLen + ")");


  // Check if OPCUA -num iDriverNumber and simulator exists
  bOpcUaExists     = FALSE;
  bSimulatorExists = FALSE;


  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    // Check if OPC UA client is already defined
    if( (ddmAllManagers[iLoop][1] == "WCCILsim")             &&
        (ddmAllManagers[iLoop][6] == "-num " + iDriverNumber) )
    {
      bSimulatorExists = TRUE;
      DebugTN("CreateOpcUaManager() -> Simulator manager with driver number: " + iDriverNumber + " already exists");
    }
  }


  if( bSimulatorExists )
  {
    // Change Simulator manager to manual
    if( fwInstallationManager_setMode("WCCILsim", "-num " + iDriverNumber, "manual") != 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Change of startup mode for simulator manager, driver number " + iDriverNumber + " failed in fwInstallationManager_setMode()", "");
      return FALSE;
    }
    else
    {
      DebugTN("CreateOpcUaManager() -> Change of startup mode for Simulator manager, driver number " + iDriverNumber + " to manual OK");
    }


    // Check if Simulator manager is running
    if( fwInstallationManager_isRunning("WCCILsim", "-num " + iDriverNumber, bIsRunning) != 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Check if Simulator manager, driver number " + iDriverNumber + " is running failled in fwInstallationManager_isRunning()", "");
      return FALSE;
    }
    else
    {
      DebugTN("CreateOpcUaManager() -> Simulator manager with driver number " + iDriverNumber + " is: " + (bIsRunning?"Started":"Stopped") );

      // If simulator manager is running, stop it and delay 5 seconds to close it.
      if( bIsRunning )
      {
        if( fwInstallationManager_command("STOP", "WCCILsim", "-num " + iDriverNumber) != 0 )
        {
          fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Error stopping simulation manager with driver number " + iDriverNumber + " in fwInstallationManager_command()", "");
          return FALSE;
        }
        DebugTN("CreateOpcUaManager() -> Simulation manager with with driver number " + iDriverNumber + " ordered to stop, waiting 5 seconds...");
        delay(5);
      }
    }
  }


  for( iLoop = 1 ; iLoop <= iLen ; iLoop++ )
  {
    // Check if OPC UA client is already defined
    if( (ddmAllManagers[iLoop][1] == "WCCOAopcua")             &&
        (ddmAllManagers[iLoop][6] == "-num " + iDriverNumber) )
    {
      bOpcUaExists = TRUE;
      DebugTN("CreateOpcUaManager() -> OPC UA manager with driver number: " + iDriverNumber + " already exists");
    }
  }


  // Create OPC UA manager if doesn't exist
  if( !bOpcUaExists )
  {
    DebugTN("CreateOpcUaManager() -> OPC UA manager with driver number: " + iDriverNumber + " doesn't exist. It will be created");

    iRes = fwInstallationManager_add("WCCOAopcua", "manual", 30, 5, 3, "-num " + iDriverNumber);
    if( iRes == 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Creation of OPC UA manager, driver number " + iDriverNumber + " failed calling fwInstallationManager_add()", "");
      return FALSE;
    }
    else
    {
      DebugTN("CreateOpcUaManager() -> OPC UA manager with driver number: " + iDriverNumber + " created OK");
    }
  }
  else
  {
    // Change OPC UA manager to manual
    if( fwInstallationManager_setMode("WCCOAopcua", "-num " + iDriverNumber, "manual") != 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Change of startup mode for OPC UA manager, driver number " + iDriverNumber + " failed in fwInstallationManager_setMode()", "");
      return FALSE;
    }
    else
    {
      DebugTN("CreateOpcUaManager() -> Change of startup mode for OPC UA manager, driver number " + iDriverNumber + " to manual OK");
    }
  }


  // Check if manager is running
  if( fwInstallationManager_isRunning("WCCOAopcua", "-num " + iDriverNumber, bIsRunning) != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Check if OPC UA manager, driver number " + iDriverNumber + " is running failled in fwInstallationManager_isRunning()", "");
    return FALSE;
  }
  DebugTN("CreateOpcUaManager() -> OPC UA manager, driver number " + iDriverNumber + " status is: " + (bIsRunning?"Started":"Stopped") );

  if( !bIsRunning )
  {
    if( fwInstallationManager_restart("WCCOAopcua", "-num " + iDriverNumber) != 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "CreateOpcUaManagers() -> Error restarting OPC UA manager, driver number " + iDriverNumber + " in fwInstallationManager_restart()", "");
      return FALSE;
    }
    DebugTN("CreateOpcUaManager() -> OPC UA manager, driver number " + iDriverNumber + " ordered to restart");
  }

  return TRUE;
}





bool ImportDplFile(string sPath, string sFile, dyn_string &exceptionInfo)
{
  int iSystemCall;
  string sCommand, sAsciiTool, sUser, sPassword, sImportLog, sImportLogError, sFullName;
  dyn_string dsConfigKey, dsConfigAnswer;


  // Generate file paths
  sFullName       = sPath + sFile + ".dpl";
  sImportLog      = getPath(LOG_REL_PATH, "", "", 1) + sFile + "_info.log";
  sImportLogError = getPath(LOG_REL_PATH, "", "", 1) + sFile + "_error.log";
  if(_WIN32)
  {
    strreplace(sFullName,       "/", "\\");
    strreplace(sImportLog,      "/", "\\");
    strreplace(sImportLogError, "/", "\\");
  }


  // Get ASCII manager binary name
  sAsciiTool = getComponentName(ASCII_COMPONENT);
  if(_WIN32)
  {
    sAsciiTool = sAsciiTool + ".exe";
  }


  // Get binary path
  sCommand = getPath(BIN_REL_PATH, sAsciiTool);
  if(_WIN32)
  {
    strreplace(sCommand, "/", "\\");
  }


  // Get user name and password
  dsConfigKey = makeDynString("userName", "password");
  paCfgReadValue(PROJ_PATH + "config/config", "general", dsConfigKey, dsConfigAnswer);
  if( (dsConfigAnswer[1]  == "")                ||
      (getUserName()      != dsConfigAnswer[1]) ||
      (!getUserPermission(4))                     ) // no default user or other user or not enough permission
  {
    fwException_raise(exceptionInfo, "ERROR", "ImportDplFile() -> Error: not default user or other user not enough permissions", "");
    return FALSE;
  }
  else
  {
    sUser     = dsConfigAnswer[1];
    sPassword = dsConfigAnswer[2];
  }
  sCommand = sCommand + " -user " + sUser + ":" + sPassword + ((sPassword != "")? " " : "");


  // Add data port and event port
  sCommand = sCommand + " -data "  + (dataHost()) [1] + ":" + dataPort();
  sCommand = sCommand + " -event " + (eventHost())[1] + ":" + eventPort();


  // Add import file
  sCommand = sCommand + " -in \"" + sFullName + "\"";


  // Add overwrite DPT and load data with alarm handling
  sCommand = sCommand + " -yes";
  sCommand = sCommand + " -inactivateAlert";


  // Add logging
  sCommand = sCommand + " -log +stderr";
  sCommand = sCommand + " -log -file > " + sImportLog + " 2> " + sImportLogError;
  
  
  //Launch the system call
  if(_WIN32)
  {
    sCommand = "cmd /c " + sCommand;
  }
  DebugTN("sCommand = ", sCommand);

  iSystemCall = system(sCommand);
  switch( iSystemCall )
  {
    case -1:
      fwException_raise(exceptionInfo, "ERROR", "ImportDplFile() -> Error: system call without arguments", "");
      return FALSE;
      break;

    case 0:
      DebugTN("ImportDplFile() -> DpList file: " + sPath + sFile + ", imported OK");
      break;

    default:
      fwException_raise(exceptionInfo, "ERROR", "ImportDplFile() -> Error: system call returned an error: " + iSystemCall, "");
      return FALSE;
      break;
  }

  return TRUE;
}





/**@brief Prepare the appropiate environment before run each test case in the suite.
This function is called each time before any test case is called.

It means:

   - Import dpList to set all Crates to the default values
   - Wait until the conditions are meet.

*/
void test_fwCaen_setup()
{
  DebugTN("test_fwCaen_setup() -> Launched...");


  bool bMute;
  int iSum, iRes;
  dyn_int diScadaCounter;
  dyn_string exceptionInfo;
  dyn_errClass err;


  // Mute Pilot OPC UA server
  DebugTN("test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> Start...");

  iRes = dpSetWait("PilotSimulator_0.Settings.OpcUaMute", TRUE);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> setting DP unknown, PilotSimulator_0.Settings.OpcUaMute", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  else
  {
    err = getLastError();
    if( dynlen(err) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> error code: " + getErrorCode(err) + ": " + getErrorText() , "");
      throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    }
    else
    {
      DebugTN("test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> Sent OK");
    }
  }

  DebugTN("test_fwCaen_setup() -> Delay 5 seconds to send command and readback the command by the WinCC OA OPC UA client");
  delay(5);

  iRes = dpGet("PilotSimulator_0.ReadBackSettings.OpcUaMute", bMute);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> readback DP unknown, PilotSimulator_0.ReadBackSettings.OpcUaMute", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  else
  {
    err = getLastError();
    if( dynlen(err) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> readback error, error code: " + getErrorCode(err) + ": " + getErrorText() , "");
      throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    }
    else
    {
      DebugTN("test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> Readback OK, bMute = " + bMute);
    }
  }

  if( bMute != TRUE )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Set Venus Caen OPC UA in mute -> Command sent, but readback failled", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  DebugTN("test_fwCaen_setup() -> Venus Caen OPC UA server mute -> OK");




  // Reset counters in Pilot OPC UA server
  DebugTN("test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> Start...");
  iRes = dpSetWait("PilotSimulator_0.Settings.CountersReset", TRUE);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> setting DP unknown, PilotSimulator_0.Settings.CountersReset", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  else
  {
    err = getLastError();
    if( dynlen(err) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> setting DP error, error code: " + getErrorCode(err) + ": " + getErrorText() , "");
      throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    }
    else
    {
      DebugTN("test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> Sent OK");
    }
  }



  DebugTN("test_fwCaen_setup() -> Delay 5 seconds to send the counter reset command and readback the command by the WinCC OA OPC UA client");
  delay(5);


  iRes = dpGet("PilotSimulator_0.Actual.ScadaCounters.Counters", diScadaCounter);
  if( iRes != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> counter DP unknown, PilotSimulator_0.Actual.ScadaCounters.Counters", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  else
  {
    err = getLastError();
    if( dynlen(err) > 0 )
    {
      fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> read counter DP error, error code: " + getErrorCode(err) + ": " + getErrorText() , "");
      throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    }
    else
    {
      DebugTN("test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> Counters read OK");
    }
  }

  iSum = dynSum(diScadaCounter);
  if( iSum != 0 )
  {
    fwException_raise(exceptionInfo, "ERROR", "test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> Error: Pilot CAEN OPC UA counters didn't reset", "");
    throw( makeError("", PRIO_SEVERE, ERR_IMPL, 86, exceptionInfo) );
    return;
  }
  DebugTN("test_fwCaen_setup() -> Reset Pilot Caen OPC UA counters -> reset OK");


  // Reset counters in WinCC OA
    // To be done...

}





/**@brief Called after each case test

*/
void test_fwCaen_teardown()
{
  DebugTN("test_fwCaen_teardown() -> Launched...");

  // Mute Pilot OPC UA server

}





/**@brief Called after all test case rutines have been run

*/
void test_fwCaen_teardownSuite()
{
  DebugTN("test_fwCaen_teardownSuite() -> Launched...");

  // Nothing to do
}





void test_fwCaen_RampOneChannel()
{
  DebugTN("test_fwCaen_RampOneChannel() -> Launched...");

  int iRes;
  float fVMon, fV0Set fV1Set, fI0Set, fI1Set, fRampUp, fRampDwn;

  // we ramp up one channel of one board:
  // use the pilot to set the proper load, no noise
  // set V0 and V1, I0 and I1, ramp speeds etc etc
  // use Pw to switch it in by software: it will ramp up to V0Set...
  // then use the pilot to go to V1Set ( VSEL, ISEL)
  //
  // we can ramp down with another method


  // pilot on pcbe13632:8890
  // caen pcbe13632:4841
  // crate=0, slot=3, channel=1
  // p_resistance = 100e8; // 100MOhm
  // p_capacitance =1e-7;  // 0.1uF
  // VSEL, ISEL=false, we are on V0/I0 therefore
  // trip time = 10 seconds hardcoded
  // Vnoise = 0
  // Inoise = 0


  DebugTN("test_fwCaen_RampOneChannel() -> Init Pilot Caen OPC UA values -> Start...");
  iRes = dpSetWait("PilotSimulator_0.Settings.ISel",                                FALSE,
                   "PilotSimulator_0.Settings.VSel",                                FALSE,
                   "PilotSimulator_0.Settings.AllChannels.PowerAllChannels",        FALSE,
                   "PilotSimulator_0.Settings.Selector.Crate",                      0,
                   "PilotSimulator_0.Settings.Selector.Board",                      3,
                   "PilotSimulator_0.Settings.Selector.Channel",                    1,
                   "PilotSimulator_0.Settings.AllChannels.ChannelLoad.Resistance",  100e8,
                   "PilotSimulator_0.Settings.AllChannels.ChannelLoad.Capacitance", 1e-7,
                   "PilotSimulator_0.Settings.SingleChannel.Noise.VoltageNoise",    0,
                   "PilotSimulator_0.Settings.SingleChannel.Noise.CurrentNoise",    0,
                   "CAEN/develSY4527/board03/channel001.settings.onOff",            0,
                   "CAEN/develSY4527/board03/channel001.settings.rDwn",             1000);
  DebugTN("test_fwCaen_RampOneChannel() -> Init Pilot Caen OPC UA values -> delay 60 seconds to assure stable point");
  delay(60);
  DebugTN("test_fwCaen_RampOneChannel() -> Init Pilot Caen OPC UA values -> Done OK");


  // p_V0Set = 15;
  // p_V1Set = 100;
  // p_I0Set = 2e-6; // mA in SI
  // p_+I1Set = 6e-6;
  // p_rup = 5;
  // p_dwn = 5;
  fV0Set   = 15;
  fV1Set   = 100;
  fI0Set   = 2e-6;
  fI1Set   = 6e-6;
  fRampUp  = 5;
  fRampDwn = 5;

  DebugTN("test_fwCaen_RampOneChannel() -> Init Venus Caen OPC UA values -> Start...");
  iRes = dpSetWait("CAEN/develSY4527/board03/channel001.settings.i0",   fI0Set,
                   "CAEN/develSY4527/board03/channel001.settings.i1",   fI1Set,
                   "CAEN/develSY4527/board03/channel001.settings.v0",   fV0Set,
                   "CAEN/develSY4527/board03/channel001.settings.v1",   fV1Set,
                   "CAEN/develSY4527/board03/channel001.settings.rUp",  fRampUp,
                   "CAEN/develSY4527/board03/channel001.settings.rDwn", fRampDwn);
  DebugTN("test_fwCaen_RampOneChannel() -> Init Venus Caen OPC UA values -> delay 5 seconds to assure stable point");
  delay(5);
  DebugTN("test_fwCaen_RampOneChannel() -> Init Venus Caen OPC UA values -> Done OK");


  DebugTN("test_fwCaen_RampOneChannel() -> Removed mute -> Start...");
  iRes = dpSetWait("PilotSimulator_0.Settings.OpcUaMute", FALSE);
  DebugTN("test_fwCaen_RampOneChannel() -> Init Venus Caen OPC UA values -> delay 5 seconds to propagate the OPC UA mute setting");
  delay(5);


  DebugTN("test_fwCaen_RampOneChannel() -> Power channel from Venus Caen OPC UA -> Start...");
  iRes = dpSetWait("CAEN/develSY4527/board03/channel001.settings.onOff", TRUE);  

  DebugTN("test_fwCaen_RampOneChannel() -> Power channel from Venus Caen OPC UA -> Delay of " + ((fV0Set/fRampUp) + 5) + " seconds for the ramping up until V0Set, I0Set");
  delay( (fV0Set/fRampUp) + 5);

  iRes = dpGet("CAEN/develSY4527/board03/channel001.actual.vMon", fVMon);
  DebugTN("test_fwCaen_RampOneChannel() -> V0Set reached? -> VMon = " + fVMon);

  DebugTN("test_fwCaen_RampOneChannel() -> V0Set reached? -> V0Set = " + fV0Set + " and VMon = " + fVMon);
  if( (fVMon < (fV0Set - (fV0Set/100*5))) ||
      (fVMon > (fV0Set + (fV0Set/100*5)))  )
  {
    assertEqual(fV0Set, fVMon, "Vmon: " + fVMon + " did't reach set point: " + fV0Set);
  }

  DebugTN("test_fwCaen_RampOneChannel() -> V1Set reached? -> Sending hardware signal to ramp from V0Set to V1Set");
  iRes = dpSet("PilotSimulator_0.Settings.VSel", TRUE);

  DebugTN("test_fwCaen_RampOneChannel() -> Power channel from Venus Caen OPC UA -> Delay of " + (((fV1Set-fVMon)/fRampUp) + 5) + " seconds for the ramping up until V1Set, I1Set");
  delay( ((fV1Set-fVMon)/fRampUp) + 5 );

  iRes = dpGet("CAEN/develSY4527/board03/channel001.actual.vMon", fVMon);
  DebugTN("test_fwCaen_RampOneChannel() -> V1Set reached? -> VMon = " + fVMon);

  DebugTN("test_fwCaen_RampOneChannel() -> V1Set reached? -> V1Set = " + fV1Set + " and VMon = " + fVMon);
  if( (fVMon < (fV1Set - (fV1Set/100*5))) ||
      (fVMon > (fV1Set + (fV1Set/100*5)))  )
  {
    assertEqual(fV1Set, fVMon, "Vmon: " + fVMon + " did't reach set point: " + fV1Set);
  }


}




/*
void test_fwCaen_RampingUp()
{
  DebugTN("test_fwCaen_RampingUp() -> Launched...");
  delay(2);
  assertEqual(3, 3, "3 and 2 are not equal");
}
*/




/*
void test_fwCaen_RampingDown()
{
  DebugTN("test_fwCaen_RampingDown() -> Launched...");
  delay(2);
}
*/




/*
void test_fwCaen_Stable_Trip()
{
  DebugTN("test_fwCaen_StableTrip() -> Launched...");
  delay(2);
}
*/




/*
void test_fwCaen_RampingUp_Trip()
{
  DebugTN("test_fwCaen_RampingUp_Trip() -> Launched...");
  delay(2);
}
*/




